-----------------------------------------------------------------------------------------------
-- Client Lua Script for DPSStats
-- Copyright (c) Foxykeep. All rights reserved
-----------------------------------------------------------------------------------------------

require "Window"


-----------------------------------------------------------------------------------------------
-- DPSStats Module Definition
-----------------------------------------------------------------------------------------------
local DPSStats = {}

local foxyLib = nil

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------

local defaultSettings = {
	isVisible = false,
	wndPosition = {50, 30, 210, 200}
}

-----------------------------------------------------------------------------------------------
-- DPSStats Initialization
-----------------------------------------------------------------------------------------------

function DPSStats:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self

    return o
end

function DPSStats:InitUserSettings()

end

function DPSStats:Init()
	local bHasConfigurateFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		"FoxyLib-1.0"
	}
    Apollo.RegisterAddon(self, bHasConfigurateFunction, strConfigureButtonText, tDependencies)
end


-----------------------------------------------------------------------------------------------
-- DPSStats Save & Restore settings
-----------------------------------------------------------------------------------------------

function DPSStats:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return nil
	end

    local tSave = {}
    tSave.isVisible = self.userSettings.isVisible
    tSave.wndPosition = foxyLib.DeepCopy(self.userSettings.wndPosition)

	return tSave
end

function DPSStats:OnRestore(eType, tSave)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end

	self.userSettings.isVisible = tSave.isVisible
	self.userSettings.wndPosition = foxyLib.DeepCopy(tSave.wndPosition)

	-- Data saved in future versions must be lazy restored (if present (~= nil), grab from tSave else
	-- grab from defaultSettings).

	self.onRestoreCalled = true
	self:SetupWndMain()
end


-----------------------------------------------------------------------------------------------
-- DPSStats OnLoad
-----------------------------------------------------------------------------------------------

function DPSStats:OnLoad()
	foxyLib = Apollo.GetPackage("FoxyLib-1.0").tPackage

	self.userSettings = foxyLib.DeepCopy(defaultSettings)

	self.onRestoreCalled = false
	self.onXmlDocLoadedCalled = false

	-- Create the form
	self.xmlDoc = XmlDoc.CreateFromFile("DPSStats.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)

	Apollo.RegisterEventHandler("VarChange_FrameCount", "OnFrame", self)
end

-----------------------------------------------------------------------------------------------
-- GotHUD OnDocLoaded
-----------------------------------------------------------------------------------------------
function DPSStats:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		-- Load the settings window
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "DPSStatsWindow", nil, self)
		if not self.wndMain then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		self.wndMain:Show(false)

		-- Register handlers for events, slash commands and timer, etc.
		Apollo.RegisterSlashCommand("DPSStats", "ToggleMainUI", self)
		Apollo.RegisterSlashCommand("dpsstats", "ToggleMainUI", self)

		self.onXmlDocLoadedCalled = true
		self:SetupWndMain()
	end
end


-----------------------------------------------------------------------------------------------
-- Wnd position
-----------------------------------------------------------------------------------------------

function DPSStats:SetupWndMain()
	if not self.onRestoreCalled or not self.onXmlDocLoadedCalled then
		return
	end

	local anchors = self.userSettings.wndPosition
	self.wndMain:SetAnchorOffsets(anchors[1], anchors[2], anchors[3], anchors[4])
end

-----------------------------------------------------------------------------------------------
-- Main loop (on every 4 frames)
-----------------------------------------------------------------------------------------------

function DPSStats:OnFrame()
	if self.userSettings.isVisible then
		-- We are in not visible. Nothing to do here. Let's just hide if we are currently visible
		if self.wndMain:IsVisible() then
			self.wndMain:Show(false)
		end
		return
	end

	local unitPlayer = GameLib.GetPlayerUnit()
	if not unitPlayer then
		-- We don't have the player object yet.
		return
	end

	-- Show the window if not currently visible
	if not self.wndMain:IsVisible() then
		self.wndMain:Show(true)
	end

	-- Save the position
	local left, top, right, bottom = self.wndMain:GetAnchorOffsets()
	self.userSettings.wndPosition  = { left, top, left+160, top+190 }
	self.wndMain:FindChild("ValueAP"):SetText(Apollo.FormatNumber(unitPlayer:GetAssaultPower(), 0, false))
	if unitPlayer:GetStrikethroughChance().nAmount >= 30 then
		self.wndMain:FindChild("ValueStrikethrough"):SetTextColor("ConTough")
	else
		self.wndMain:FindChild("ValueStrikethrough"):SetTextColor("UI_WindowTextDefault")
	end
	self.wndMain:FindChild("ValueStrikethrough"):SetText(Apollo.FormatNumber(unitPlayer:GetStrikethroughChance().nAmount, 2, true) .. '%')
	if unitPlayer:GetCritChance().nAmount >= 30 then
		self.wndMain:FindChild("ValueCHC"):SetTextColor("conTough")
	else
		self.wndMain:FindChild("ValueCHC"):SetTextColor("UI_WindowTextDefault")
	end
	self.wndMain:FindChild("ValueCHC"):SetText(Apollo.FormatNumber(unitPlayer:GetCritChance().nAmount, 2, true) .. '%')
	if unitPlayer:GetCritSeverity().nAmount >= 250 then
		self.wndMain:FindChild("ValueCHS"):SetTextColor("ConTough")
	else
		self.wndMain:FindChild("ValueCHS"):SetTextColor("UI_WindowTextDefault")
	end
	self.wndMain:FindChild("ValueCHS"):SetText(Apollo.FormatNumber(unitPlayer:GetCritSeverity().nAmount, 2, true) .. '%')
	if unitPlayer:GetMultiHitChance().nAmount >= 60 then
		self.wndMain:FindChild("ValueMHC"):SetTextColor("ConTough")
	else
		self.wndMain:FindChild("ValueMHC"):SetTextColor("UI_WindowTextDefault")
	end
	self.wndMain:FindChild("ValueMHC"):SetText(Apollo.FormatNumber(unitPlayer:GetMultiHitChance().nAmount, 2, true) .. '%')
	if unitPlayer:GetMultiHitAmount().nAmount >= 100 then
		self.wndMain:FindChild("ValueMHS"):SetTextColor("ConTough")
	else
		self.wndMain:FindChild("ValueMHS"):SetTextColor("UI_WindowTextDefault")
	end
	self.wndMain:FindChild("ValueMHS"):SetText(Apollo.FormatNumber(unitPlayer:GetMultiHitAmount().nAmount, 2, true) .. '%')
	if unitPlayer:GetVigor().nAmount >= 30 then
		self.wndMain:FindChild("ValueVigor"):SetTextColor("ConTough")
	else
		self.wndMain:FindChild("ValueVigor"):SetTextColor("UI_WindowTextDefault")
	end
	self.wndMain:FindChild("ValueVigor"):SetText(Apollo.FormatNumber(unitPlayer:GetVigor().nAmount, 2, true) .. '%')
	if unitPlayer:GetArmorPierce().nAmount >= 100 then
		self.wndMain:FindChild("ValueArmorPierce"):SetTextColor("ConTough")
	else
		self.wndMain:FindChild("ValueArmorPierce"):SetTextColor("UI_WindowTextDefault")
	end
	self.wndMain:FindChild("ValueArmorPierce"):SetText(Apollo.FormatNumber(unitPlayer:GetArmorPierce().nAmount, 2, true) .. '%')
end


-----------------------------------------------------------------------------------------------
-- Toggle visibility
-----------------------------------------------------------------------------------------------

function DPSStats:ToggleMainUI()
	self.userSettings.isVisible = not self.userSettings.isVisible
end

-----------------------------------------------------------------------------------------------
-- DPSStats Instance
-----------------------------------------------------------------------------------------------
local DPSStatsInst = DPSStats:new()
DPSStatsInst:Init()